import { ICommandPalette, WidgetTracker, showDialog, Dialog, MainAreaWidget } from '@jupyterlab/apputils';
import { ILauncher } from '@jupyterlab/launcher';
import { IMainMenu } from '@jupyterlab/mainmenu';
import { IFileBrowserFactory } from '@jupyterlab/filebrowser';
import { TensorboardManager } from './manager';
import { IRunningSessionManagers } from '@jupyterlab/running';
import { toArray } from '@lumino/algorithm';
import { LabIcon } from '@jupyterlab/ui-components';
import tensorboardSvgStr from '../style/tensorboard.svg';
import { TensorboardTabReactWidget } from './biz/widget';
import { CommandIDs } from './commands';
export const tensorboardIcon = new LabIcon({
    name: 'jupyterlab-tensorboard-p:tensorboard',
    svgstr: tensorboardSvgStr
});
/**
 * Initialization data for the tensorboard extension.
 */
const extension = {
    id: 'tensorboard',
    requires: [ICommandPalette, IFileBrowserFactory],
    optional: [ILauncher, IMainMenu, IRunningSessionManagers],
    autoStart: true,
    activate
};
export default extension;
async function activate(app, palette, browserFactory, launcher, menu, runningSessionManagers) {
    const manager = new TensorboardManager();
    const namespace = 'tensorboard';
    const tracker = new WidgetTracker({
        namespace
    });
    addCommands(app, manager, tracker, browserFactory, launcher, menu);
    if (runningSessionManagers) {
        await addRunningSessionManager(runningSessionManagers, app, manager);
    }
    palette.addItem({ command: CommandIDs.inputDirect, category: 'Tensorboard' });
    return tracker;
}
// Running Kernels and Terminals，The coin-like tab
function addRunningSessionManager(managers, app, manager) {
    class RunningTensorboard {
        constructor(model, manager) {
            this._model = model;
            this.manager = manager;
        }
        open() {
            app.commands.execute(CommandIDs.open, { modelName: this._model.name });
        }
        icon() {
            return tensorboardIcon;
        }
        label() {
            return `${this._model.name}:${this.manager.formatDir(this._model.logdir)}`;
        }
        shutdown() {
            app.commands.execute(CommandIDs.close, { tb: this._model });
            return manager.shutdown(this._model.name);
        }
    }
    return manager.getStaticConfigPromise.then(() => {
        managers.add({
            name: 'Tensorboard',
            running: () => toArray(manager.running()).map(model => new RunningTensorboard(model, manager)),
            shutdownAll: () => manager.shutdownAll(),
            refreshRunning: () => manager.refreshRunning(),
            runningChanged: manager.runningChanged
        });
    });
}
/**
 * Add the commands for the tensorboard.
 */
export function addCommands(app, manager, tracker, browserFactory, launcher, menu) {
    const { commands, serviceManager } = app;
    commands.addCommand(CommandIDs.open, {
        execute: args => {
            // if select certain
            let modelName = args['modelName'];
            const copy = args['copy'];
            const currentCWD = browserFactory.defaultBrowser.model.path;
            let widget = null;
            // step1: find an opened widget
            if (!modelName) {
                widget = tracker.find(widget => {
                    return (manager.formatDir(widget.content.currentLogDir || '') === manager.formatDir(currentCWD));
                });
            }
            else if (!copy) {
                widget = tracker.find(value => {
                    var _a;
                    return ((_a = value.content.currentTensorBoardModel) === null || _a === void 0 ? void 0 : _a.name) === modelName;
                });
            }
            // default we have only one tensorboard
            if (widget) {
                app.shell.activateById(widget.id);
                return widget;
            }
            else {
                // step2: try find opened backend widgets
                if (!modelName) {
                    const runningTensorboards = [...toArray(manager.running())];
                    // hint: Using runningTensorboards directly may cause setState to fail to respond
                    for (const model of runningTensorboards) {
                        if (manager.formatDir(model.logdir) === manager.formatDir(currentCWD)) {
                            modelName = model.name;
                        }
                    }
                }
                const tabReact = new TensorboardTabReactWidget({
                    browserFactory,
                    tensorboardManager: manager,
                    app,
                    createdModelName: modelName
                });
                const tabWidget = new MainAreaWidget({ content: tabReact });
                tracker.add(tabWidget);
                app.shell.add(tabWidget, 'main', {
                    mode: copy ? 'split-right' : undefined
                });
                app.shell.activateById(tabWidget.id);
                return tabWidget;
            }
        }
    });
    commands.addCommand(CommandIDs.openDoc, {
        execute: args => {
            window.open('https://github.com/HFAiLab/jupyterlab_tensorboard_pro');
        }
    });
    commands.addCommand(CommandIDs.close, {
        execute: args => {
            const model = args['tb'];
            tracker.forEach(widget => {
                if (widget.content.currentTensorBoardModel &&
                    widget.content.currentTensorBoardModel.name === model.name) {
                    widget.dispose();
                    widget.close();
                }
            });
        }
    });
    commands.addCommand(CommandIDs.inputDirect, {
        label: () => 'Open TensorBoard',
        execute: args => {
            return app.commands.execute(CommandIDs.open);
        }
    });
    commands.addCommand(CommandIDs.createNew, {
        label: args => (args['isPalette'] ? 'New TensorBoard' : 'TensorBoard'),
        caption: 'Start a new tensorboard',
        icon: args => (args['isPalette'] ? undefined : tensorboardIcon),
        execute: args => {
            const cwd = args['cwd'] || browserFactory.defaultBrowser.model.path;
            const logdir = typeof args['logdir'] === 'undefined' ? cwd : args['logdir'];
            return serviceManager.contents.get(logdir, { type: 'directory' }).then(dir => {
                // Try to open the session panel to make it easier for users to observe more active tensorboard instances
                try {
                    app.shell.activateById('jp-running-sessions');
                }
                catch (e) {
                    // do nothing
                }
                app.commands.execute(CommandIDs.open);
            }, () => {
                // no such directory.
                return showDialog({
                    title: 'Cannot create tensorboard.',
                    body: 'Directory not found',
                    buttons: [Dialog.okButton()]
                });
            });
        }
    });
    if (launcher) {
        launcher.add({
            command: CommandIDs.createNew,
            category: 'Other',
            rank: 2
        });
    }
    if (menu) {
        menu.fileMenu.newMenu.addGroup([
            {
                command: CommandIDs.createNew
            }
        ], 30);
    }
}
